@@ -206,8 +206,11 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
             my_env["PATH"] = omhome_bin + os.pathsep + my_env["PATH"]
             self._omc_process = subprocess.Popen(self._omc_command, stdout=self._omc_log_file, stderr=self._omc_log_file, env=my_env)
         else:
+            # set the user environment variable so omc running from wsgi has the same user as OMPython
+            my_env = os.environ.copy()
+            my_env["USER"] = self._currentUser
             # Because we spawned a shell, and we need to be able to kill OMC, create a new process group for this
-            self._omc_process = subprocess.Popen(self._omc_command, shell=True, stdout=self._omc_log_file, stderr=self._omc_log_file, preexec_fn=os.setsid)
+            self._omc_process = subprocess.Popen(self._omc_command, shell=True, stdout=self._omc_log_file, stderr=self._omc_log_file, env=my_env, preexec_fn=os.setsid)
         if self._docker:
           for i in range(0,40):
             try:
@@ -528,7 +531,7 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
                                  str(builtin).lower(), str(showProtected).lower()))
         else:
             value = self.ask('getClassNames',
-                             'recursive={1}, qualified={2}, sort={3}, builtin={4}, showProtected={5}'.format(
+                             'recursive={0}, qualified={1}, sort={2}, builtin={3}, showProtected={4}'.format(
                                  str(recursive).lower(), str(qualified).lower(), str(sort).lower(),
                                  str(builtin).lower(), str(showProtected).lower()))
         return value
@@ -698,6 +701,7 @@ class OMCSessionZMQ(OMCSessionHelper, OMCSessionBase):
         # set omc executable path and args
         self._set_omc_command([
                                "--interactive=zmq",
+                               "--locale=C",
                                "-z={0}".format(self._random_string)
                                ])
         # start up omc executable, which is waiting for the ZMQ connection
@@ -784,7 +788,7 @@ class OMCSessionZMQ(OMCSessionHelper, OMCSessionBase):
 
 
 class ModelicaSystem(object):
-    def __init__(self, fileName=None, modelName=None, lmodel=[], useCorba=False, commandLineOptions=None):  # 1
+    def __init__(self, fileName=None, modelName=None, lmodel=[], useCorba=False, commandLineOptions=None, variableFilter=None):  # 1
         """
         "constructor"
         It initializes to load file and build a model, generating object, exe, xml, mat, and json files. etc. It can be called :
@@ -794,7 +798,6 @@ class ModelicaSystem(object):
         Note: If the model file is not in the current working directory, then the path where file is located must be included together with file name. Besides, if the Modelica model contains several different models within the same package, then in order to build the specific model, in second argument, user must put the package name with dot(.) followed by specific model name.
         ex: myModel = ModelicaSystem("ModelicaModel.mo", "modelName")
         """
-
         if fileName is None and modelName is None and not lmodel:  # all None
             if useCorba:
                 self.getconn = OMCSession()
@@ -802,10 +805,7 @@ class ModelicaSystem(object):
                 self.getconn = OMCSessionZMQ()
             return
 
-        if fileName is None:
-            return "File does not exist"
         self.tree = None
-
         self.quantitiesList=[]
         self.paramlist={}
         self.inputlist={}
@@ -814,19 +814,22 @@ class ModelicaSystem(object):
         self.simulateOptions={}
         self.overridevariables={}
         self.simoptionsoverride={}
-        self.linearOptions={'startTime':0.0, 'stopTime': 1.0, 'numberOfIntervals':500, 'stepSize':0.002, 'tolerance':1e-8}
+        self.linearOptions={'startTime':0.0, 'stopTime': 1.0, 'stepSize':0.002, 'tolerance':1e-8}
         self.optimizeOptions={'startTime':0.0, 'stopTime': 1.0, 'numberOfIntervals':500, 'stepSize':0.002, 'tolerance':1e-8}
-        self.linearquantitiesList = []  # linearization  quantity list
-        self.linearparameters={}
         self.linearinputs = []  # linearization input list
         self.linearoutputs = []  # linearization output list
         self.linearstates = []  # linearization  states list
+        self.tempdir = ""
 
         if useCorba:
             self.getconn = OMCSession()
         else:
             self.getconn = OMCSessionZMQ()
 
+        ## needed for properly deleting the OMCSessionZMQ
+        self._omc_log_file = self.getconn._omc_log_file
+        self._omc_process = self.getconn._omc_process
+
         ## set commandLineOptions if provided by users
         if commandLineOptions is not None:
             exp="".join(["setCommandLineOptions(","\"",commandLineOptions,"\"",")"])
@@ -838,80 +841,106 @@ class ModelicaSystem(object):
         self.fileName = fileName  # Model file/package name
         self.inputFlag = False  # for model with input quantity
         self.simulationFlag = False  # if the model is simulated?
-        self.linearizationFlag = False
         self.outputFlag = False
         self.csvFile = ''  # for storing inputs condition
         self.resultfile="" # for storing result file
-        if not os.path.exists(self.fileName):  # if file does not eixt
+        self.variableFilter = variableFilter
+
+        if fileName is not None and  not os.path.exists(self.fileName):  # if file does not eixt
             print("File Error:" + os.path.abspath(self.fileName) + " does not exist!!!")
             return
 
-        (head, tail) = os.path.split(self.fileName)  # to store directory/path and file)
-        self.currDir = os.getcwd()
-        self.modelDir = head
-        self.fileName_ = tail
+        ## set default command Line Options for linearization as
+        ## linearize() will use the simulation executable and runtime
+        ## flag -l to perform linearization
+        self.getconn.sendExpression("setCommandLineOptions(\"--linearizationDumpLanguage=python\")")
+        self.getconn.sendExpression("setCommandLineOptions(\"--generateSymbolicLinearization\")")
 
-        if not self.modelDir:
-            file_ = os.path.exists(self.fileName_)
-            if (file_):  # execution from path where file is located
-                self.__loadingModel()
-            else:
-                print("Error: File does not exist!!!")
+        self.setTempDirectory()
 
-        else:
-            os.chdir(self.modelDir)
-            file_ = os.path.exists(self.fileName_)
-            self.model = self.fileName_[:-3]
-            if (self.fileName_):  # execution from different path
-                os.chdir(self.currDir)
-                self.__loadingModel()
-            else:
-                print("Error: File does not exist!!!")
+        if fileName is not None:
+            self.loadFile()
+
+        ## allow directly loading models from MSL without fileName
+        if fileName is None and modelName is not None:
+            self.loadLibrary()
+
+        self.buildModel()
 
     def __del__(self):
-        if self.getconn is not None:
-            self.requestApi('quit')
+        OMCSessionBase.__del__(self)
+
+    def setCommandLineOptions(self):
+        ## set commandLineOptions if provided by users
+        if commandLineOptions is not None:
+            exp="".join(["setCommandLineOptions(","\"",commandLineOptions,"\"",")"])
+            cmdexp = self.getconn.sendExpression(exp)
+            if not cmdexp:
+                return print(self.getconn.sendExpression("getErrorString()"))
+
+    def loadFile(self):
+        # load file
+        loadFileExp="".join(["loadFile(","\"",self.fileName,"\"",")"]).replace("\\","/")
+        loadMsg = self.getconn.sendExpression(loadFileExp)
+        if not loadMsg:
+            return print(self.getconn.sendExpression("getErrorString()"))
 
     # for loading file/package, loading model and building model
-    def __loadingModel(self):
-        # load file
-        loadfileError = ''
-        loadfileResult = self.requestApi("loadFile", self.fileName)
-        loadfileError = self.requestApi("getErrorString")
-
-        # print the notification to users
-        if(loadfileResult==True and loadfileError):
-            print(loadfileError)
-
-        if (loadfileResult==False):
-            specError = 'Parser error: Unexpected token near: optimization (IDENT)'
-            if specError in loadfileError:
-                self.requestApi("setCommandLineOptions", '"+g=Optimica"')
-                self.requestApi("loadFile", self.fileName)
-            else:
-                print('loadFile Error: ' + loadfileError)
-                return
-
+    def loadLibrary(self):
         # load Modelica standard libraries or Modelica files if needed
         for element in self.lmodel:
             if element is not None:
                 loadmodelError = ''
-                if element.endswith(".mo"):
-                    loadModelResult = self.requestApi("loadFile", element)
-                    loadmodelError = self.requestApi('getErrorString')
+                if isinstance(element, str):
+                    if element.endswith(".mo"):
+                        loadModelResult = self.requestApi("loadFile", element)
+                        if not loadModelResult:
+                            loadmodelError = self.requestApi('getErrorString')
+                    else:
+                        loadModelResult = self.requestApi("loadModel", element)
+                        if not loadModelResult:
+                            loadmodelError = self.requestApi('getErrorString')
+                elif isinstance(element, tuple):
+                    if not element[1]:
+                        libname = "".join(["loadModel(", element[0], ")"])
+                    else:
+                        libname = "".join(["loadModel(", element[0], ", ", "{", "\"", element[1], "\"", "}", ")"])
+                    loadModelResult = self.sendExpression(libname)
+                    if not loadModelResult:
+                        loadmodelError = self.sendExpression("getErrorString()")
                 else:
-                    loadModelResult = self.requestApi("loadModel", element)
-                    loadmodelError = self.requestApi('getErrorString')
+                    print("| info | loadLibrary() failed, Unknown type detected: ", element , " is of type ",  type(element), ", The following patterns are supported\n1)[\"Modelica\"]\n2)[(\"Modelica\",\"3.2.3\"), \"PowerSystems\"]\n")
                 if loadmodelError:
                     print(loadmodelError)
-        self.buildModel()
 
-    def buildModel(self):
+    def setTempDirectory(self):
+        # create a unique temp directory for each session and build the model in that directory
+        self.tempdir = tempfile.mkdtemp()
+        if not os.path.exists(self.tempdir):
+            return print(self.tempdir, " cannot be created")
+
+        exp="".join(["cd(","\"",self.tempdir,"\"",")"]).replace("\\","/")
+        self.getconn.sendExpression(exp)
+
+    def getWorkDirectory(self):
+        return self.tempdir
+
+    def buildModel(self, variableFilter=None):
+        if variableFilter is not None:
+            self.variableFilter = variableFilter
+
+        if self.variableFilter is not None:
+            varFilter = "variableFilter=" + "\"" + self.variableFilter + "\""
+        else:
+            varFilter = "variableFilter=" +  "\".*""\""
+        # print(varFilter)
         # buildModelResult=self.getconn.sendExpression("buildModel("+ mName +")")
-        buildModelResult = self.requestApi("buildModel", self.modelName)
+        buildModelResult = self.requestApi("buildModel", self.modelName, properties=varFilter)
         buildModelError = self.requestApi("getErrorString")
-        if ('' in buildModelResult):
+        # Issue #145. Always print the getErrorString since it might contains build warnings.
+        if buildModelError:
             print(buildModelError)
+        if ('' in buildModelResult):
             return
         self.xmlFile=os.path.join(os.path.dirname(buildModelResult[0]),buildModelResult[1]).replace("\\","/")
         self.xmlparse()
@@ -949,48 +978,44 @@ class ModelicaSystem(object):
                 self.simulateOptions["stepSize"] = attr.get('stepSize')
                 self.simulateOptions["tolerance"] = attr.get('tolerance')
                 self.simulateOptions["solver"] = attr.get('solver')
+                self.simulateOptions["outputFormat"] = attr.get('outputFormat')
 
             for sv in rootCQ.iter('ScalarVariable'):
                 scalar={}
                 scalar["name"] = sv.get('name')
-                scalar["changable"] = sv.get('isValueChangeable')
+                scalar["changeable"] = sv.get('isValueChangeable')
                 scalar["description"] = sv.get('description')
                 scalar["variability"] = sv.get('variability')
                 scalar["causality"] = sv.get('causality')
                 scalar["alias"] = sv.get('alias')
                 scalar["aliasvariable"] = sv.get('aliasVariable')
-                ch = sv.getchildren()
+                ch = list(sv)
                 start = None
+                min = None
+                max = None
                 for att in ch:
                     start = att.get('start')
+                    min = att.get('min')
+                    max = att.get('max')
                 scalar["start"] =start
+                scalar["min"] = min
+                scalar["max"] = max
 
-                if(self.linearizationFlag==False):
-                    if(scalar["variability"]=="parameter"):
-                        self.paramlist[scalar["name"]]=scalar["start"]
-                    if(scalar["variability"]=="continuous"):
-                        self.continuouslist[scalar["name"]]=scalar["start"]
-                    if(scalar["causality"]=="input"):
-                        self.inputlist[scalar["name"]]=scalar["start"]
-                    if(scalar["causality"]=="output"):
-                        self.outputlist[scalar["name"]]=scalar["start"]
+                if(scalar["variability"]=="parameter"):
+                    if scalar["name"] in self.overridevariables:
+                        self.paramlist[scalar["name"]] = self.overridevariables[scalar["name"]]
+                    else:
+                        self.paramlist[scalar["name"]] = scalar["start"]
+                if(scalar["variability"]=="continuous"):
+                    self.continuouslist[scalar["name"]]=scalar["start"]
+                if(scalar["causality"]=="input"):
+                    self.inputlist[scalar["name"]]=scalar["start"]
+                if(scalar["causality"]=="output"):
+                    self.outputlist[scalar["name"]]=scalar["start"]
 
-                if(self.linearizationFlag==True):
-                    if(scalar["variability"]=="parameter"):
-                        self.linearparameters[scalar["name"]]=scalar["start"]
-                    if(scalar["alias"]=="alias"):
-                        name=scalar["name"]
-                        if (name[1] == 'x'):
-                            self.linearstates.append(name[3:-1])
-                        if (name[1] == 'u'):
-                            self.linearinputs.append(name[3:-1])
-                        if (name[1] == 'y'):
-                            self.linearoutputs.append(name[3:-1])
-                    self.linearquantitiesList.append(scalar)
-                else:
-                    self.quantitiesList.append(scalar)
+                self.quantitiesList.append(scalar)
         else:
-            print("Error: ! XML file not generated")
+            print("Error: ! XML file not generated: " + self.xmlFile)
             return
 
 
@@ -1183,7 +1208,7 @@ class ModelicaSystem(object):
             return ([self.optimizeOptions.get(x,"NotExist") for x in names])
 
     # to simulate or re-simulate model
-    def simulate(self,resultfile=None,simflags=None):  # 11
+    def simulate(self, resultfile=None, simflags=None, verbose=True):  # 11
         """
         This method simulates model according to the simulation options.
         usage
@@ -1193,7 +1218,7 @@ class ModelicaSystem(object):
         """
         if(resultfile is None):
             r=""
-            self.resultfile = "".join([self.modelName, "_res.mat"])
+            self.resultfile = os.path.join(self.tempdir, self.modelName + "_res.mat").replace("\\", "/")
         else:
             r=" -r=" + resultfile
             self.resultfile = resultfile
@@ -1202,13 +1227,19 @@ class ModelicaSystem(object):
         if(simflags is None):
             simflags=""
         else:
-            simflags=" " + simflags;
+            simflags=" " + simflags
 
+        overrideFile = os.path.join(self.tempdir, '{}.{}'.format(self.modelName + "_override", "txt")).replace("\\", "/")
         if (self.overridevariables or self.simoptionsoverride):
             tmpdict=self.overridevariables.copy()
             tmpdict.update(self.simoptionsoverride)
-            values1 = ','.join("%s=%s" % (key, val) for (key, val) in list(tmpdict.items()))
-            override =" -override=" + values1
+            # write to override file
+            file = open(overrideFile, "w")
+            for (key, value) in tmpdict.items():
+                name = key + "=" + value + "\n"
+                file.write(name)
+            file.close()
+            override =" -overrideFile=" + overrideFile
         else:
             override =""
 
@@ -1227,33 +1258,40 @@ class ModelicaSystem(object):
                 if val[0][0] < float(self.simulateOptions["startTime"]):
                     print('Input time value is less than simulation startTime for inputs', i)
                     return
-            self.__simInput()  # create csv file
+            self.createCSVData()  # create csv file
             csvinput=" -csvInput=" + self.csvFile
         else:
             csvinput=""
 
         if (platform.system() == "Windows"):
-            getExeFile = os.path.join(os.getcwd(), '{}.{}'.format(self.modelName, "exe")).replace("\\", "/")
+            getExeFile = os.path.join(self.tempdir, '{}.{}'.format(self.modelName, "exe")).replace("\\", "/")
         else:
-            getExeFile = os.path.join(os.getcwd(), self.modelName).replace("\\", "/")
-
+            getExeFile = os.path.join(self.tempdir, self.modelName).replace("\\", "/")
+        currentDir = os.getcwd()
         if (os.path.exists(getExeFile)):
             cmd = getExeFile + override + csvinput + r + simflags
             #print(cmd)
+            os.chdir(self.tempdir)
             if (platform.system() == "Windows"):
-                omhome = os.path.join(os.environ.get("OPENMODELICAHOME"), 'bin').replace("\\", "/")
+                omhome = os.path.join(os.environ.get("OPENMODELICAHOME"))
+                dllPath = os.path.join(omhome, "bin").replace("\\", "/") + os.pathsep + os.path.join(omhome, "lib/omc").replace("\\", "/") + os.pathsep + os.path.join(omhome, "lib/omc/cpp").replace("\\", "/") +  os.pathsep + os.path.join(omhome, "lib/omc/omsicpp").replace("\\", "/")
                 my_env = os.environ.copy()
-                my_env["PATH"] = omhome + os.pathsep + my_env["PATH"]
-                p = subprocess.Popen(cmd, env=my_env)
+                my_env["PATH"] = dllPath + os.pathsep + my_env["PATH"]
+                if not verbose:
+                    p = subprocess.Popen(cmd, env=my_env, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
+                else:
+                    p = subprocess.Popen(cmd, env=my_env)
                 p.wait()
                 p.terminate()
             else:
-                os.system(cmd)
+                if not verbose:
+                    p = subprocess.Popen(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
+                else:
+                    p = subprocess.Popen(cmd)
+            os.chdir(currentDir)
             self.simulationFlag = True
-
         else:
-            raise Exception("Error: application file not generated yet")
-
+            raise Exception("Error: Application file path not found: " +  getExeFile)
 
     # to extract simulation results
     def getSolutions(self, varList=None, resultfile=None):  # 12
@@ -1275,17 +1313,16 @@ class ModelicaSystem(object):
 
         # check for result file exits
         if (not os.path.exists(resFile)):
-            print("Error: Result file does not exist")
+            print("Error: Result file does not exist " + resFile)
             return
             #exit()
         else:
+            resultVars = self.getconn.sendExpression("readSimulationResultVars(\"" + resFile + "\")")
+            self.getconn.sendExpression("closeSimulationResultFile()")
             if (varList == None):
-                # validSolution = ['time'] + self.__getInputNames() + self.__getContinuousNames() + self.__getParameterNames()
-                validSolution = self.getconn.sendExpression("readSimulationResultVars(\"" + resFile + "\")")
-                self.getconn.sendExpression("closeSimulationResultFile()")
-                return validSolution
+                return resultVars
             elif (isinstance(varList,str)):
-                if (varList not in [l["name"] for l in self.quantitiesList] and varList!="time"):
+                if (varList not in resultVars and varList!="time"):
                     print('!!! ', varList, ' does not exist\n')
                     return
                 exp = "readSimulationResult(\"" + resFile + '",{' + varList + "})"
@@ -1299,7 +1336,7 @@ class ModelicaSystem(object):
                 for v in varList:
                     if v == "time":
                         continue
-                    if v not in [l["name"] for l in self.quantitiesList]:
+                    if v not in resultVars:
                         print('!!! ', v, ' does not exist\n')
                         return
                 variables = ",".join(varList)
@@ -1316,7 +1353,7 @@ class ModelicaSystem(object):
         elif(isinstance(name,list)):
             return [x.replace(" ","") for x in name]
 
-    def setMethodHelper(self,args1,args2,args3,args4=None):
+    def setMethodHelper(self,args1,args2,args3,args4=None,verbose=None):
         """
         Helper function for setParameter(),setContinuous(),setSimulationOptions(),setLinearizationOption(),setOptimizationOption()
         args1 - string or list of string given by user
@@ -1328,23 +1365,32 @@ class ModelicaSystem(object):
             args1=self.strip_space(args1)
             value=args1.split("=")
             if value[0] in args2:
-                args2[value[0]]=value[1]
-                if(args4!=None):
-                    args4[value[0]]=value[1]
+                if (args3 == "parameter" and self.isParameterChangeable(value[0], value[1], verbose)):
+                    args2[value[0]]=value[1]
+                    if(args4!=None):
+                        args4[value[0]]=value[1]
+                elif (args3 != "parameter"):
+                    args2[value[0]]=value[1]
+                    if(args4!=None):
+                        args4[value[0]]=value[1]
             else:
-                print(value[0], "!is not a", args3 , "variable")
+                print("\"" + value[0] + "\"" + " is not a" +  args3 + " variable")
                 return
         elif(isinstance(args1,list)):
             args1=self.strip_space(args1)
             for var in args1:
                 value=var.split("=")
                 if value[0] in args2:
-                    args2[value[0]]=value[1]
-                    if(args4!=None):
-                        args4[value[0]]=value[1]
+                    if (args3 == "parameter" and self.isParameterChangeable(value[0], value[1], verbose)):
+                        args2[value[0]]=value[1]
+                        if(args4!=None):
+                            args4[value[0]]=value[1]
+                    elif (args3 != "parameter"):
+                        args2[value[0]]=value[1]
+                        if(args4!=None):
+                            args4[value[0]]=value[1]
                 else:
-                    print(value[0], "!is not a", args3 ,"variable")
-                    return
+                    print("\"" + value[0] + "\"" + " is not a "+ args3 + " variable")
 
     def setContinuous(self, cvals):  # 13
         """
@@ -1356,7 +1402,7 @@ class ModelicaSystem(object):
         """
         return self.setMethodHelper(cvals,self.continuouslist,"continuous",self.overridevariables)
 
-    def setParameters(self, pvals):  # 14
+    def setParameters(self, pvals, verbose=True):  # 14
         """
         This method is used to set parameter values. It can be called:
         with a sequence of parameter name and assigning corresponding value as arguments as show in the example below:
@@ -1364,7 +1410,15 @@ class ModelicaSystem(object):
         >>> setParameters("Name=value")
         >>> setParameters(["Name1=value1","Name2=value2"])
         """
-        return self.setMethodHelper(pvals,self.paramlist,"parameter",self.overridevariables)
+        return self.setMethodHelper(pvals,self.paramlist,"parameter",self.overridevariables, verbose)
+
+    def isParameterChangeable(self, name, value, verbose):
+        q = self.getQuantities(name)
+        if (q[0]["changeable"] == "false"):
+            if verbose:
+                print("| info |  setParameters() failed : It is not possible to set the following signal " + "\"" + name + "\"" + ", It seems to be structural, final, protected or evaluated or has a non-constant binding, use sendExpression(setParameterValue("+ self.modelName + ", " + name + ", " + value + "), parsed=false)" + " and rebuild the model using buildModel() API")
+            return False
+        return True
 
     def setSimulationOptions(self, simOptions):  # 16
         """
@@ -1450,12 +1504,20 @@ class ModelicaSystem(object):
                 return
 
     # To create csv file for inputs
-    def __simInput(self):
+    def createCSVData(self):
         sl = list()  # Actual timestamps
         skip = False
-        #inp = list()
-        #inp = deepcopy(self.__getInputValues())
-        inp = deepcopy(list(self.inputlist.values()))
+
+        ## check for NONE in input list and replace with proper data (e.g) [(startTime, 0.0), (stopTime, 0.0)]
+        tmpinputlist = {}
+        for (key, value) in self.inputlist.items():
+            if (value is None):
+                tmpinputlist[key] = [(float(self.simulateOptions["startTime"]), 0.0),(float(self.simulateOptions["stopTime"]), 0.0)]
+            else:
+                tmpinputlist[key] = value
+
+        inp = list(tmpinputlist.values())
+
         for i in inp:
             cl = list()
             el = list()
@@ -1543,10 +1605,11 @@ class ModelicaSystem(object):
             a = ("%s,%s" % (str(float(sl[i])), ",".join(list(str(float(inppp[i])) for inppp in interpolated_inputs_all)))) + ',0'
             l.append(a)
 
-        self.csvFile = '{}.csv'.format(self.modelName)
+        self.csvFile = os.path.join(self.tempdir, '{}.{}'.format(self.modelName, "csv")).replace("\\", "/")
         with open(self.csvFile, "w") as f:
             writer = csv.writer(f, delimiter='\n')
             writer.writerow(l)
+        f.close()
 
     # to convert Modelica model to FMU
     def convertMo2Fmu(self, version="2.0", fmuType="me_cs", fileNamePrefix="<default>", includeResources=True):  # 19
@@ -1558,19 +1621,21 @@ class ModelicaSystem(object):
         >>> convertMo2Fmu()
         >>> convertMo2Fmu(version="2.0", fmuType="me|cs|me_cs", fileNamePrefix="<default>", includeResources=true)
         """
-        convertMo2FmuError = ''
-        if fileNamePrefix == "<default":
+
+        if fileNamePrefix == "<default>":
           fileNamePrefix = self.modelName
         if includeResources:
           includeResourcesStr = "true"
         else:
           includeResourcesStr = "false"
         properties = 'version="{}", fmuType="{}", fileNamePrefix="{}", includeResources={}'.format(version, fmuType, fileNamePrefix,includeResourcesStr)
-        translateModelFMUResult = self.requestApi('translateModelFMU', self.modelName, properties)
-        if convertMo2FmuError:
-            print(convertMo2FmuError)
+        fmu = self.requestApi('buildModelFMU', self.modelName, properties)
 
-        return translateModelFMUResult
+        ## report proper error message
+        if not os.path.exists(fmu):
+            return print(self.getconn.sendExpression("getErrorString()"))
+
+        return fmu
 
     # to convert FMU to Modelica model
     def convertFmu2Mo(self, fmuName):  # 20
@@ -1580,13 +1645,14 @@ class ModelicaSystem(object):
         usage
         >>> convertFmu2Mo("c:/BouncingBall.Fmu")
         """
-        convertFmu2MoError = ''
-        importResult = self.requestApi('importFMU', fmuName)
-        convertFmu2MoError = self.requestApi('getErrorString')
-        if convertFmu2MoError:
-            print(convertFmu2MoError)
 
-        return importResult
+        fileName = self.requestApi('importFMU', fmuName)
+
+        ## report proper error message
+        if not os.path.exists(fileName):
+            return print(self.getconn.sendExpression("getErrorString()"))
+
+        return fileName
 
     # to optimize model
     def optimize(self):  # 21
@@ -1608,66 +1674,108 @@ class ModelicaSystem(object):
         return optimizeResult
 
     # to linearize model
-    def linearize(self):  # 22
+    def linearize(self, lintime = None, simflags= None):  # 22
         """
         This method linearizes model according to the linearized options. This will generate a linear model that consists of matrices A, B, C and D.  It can be called:
         only without any arguments
         usage
         >>> linearize()
         """
-        try:
-            self.getconn.sendExpression("setCommandLineOptions(\"+generateSymbolicLinearization\")")
-            properties = ','.join("%s=%s" % (key, val) for (key, val) in list(self.linearOptions.items()))
-            if (self.overridevariables):
-                values = ','.join("%s=%s" % (key, val) for (key, val) in list(self.overridevariables.items()))
-                override ="-override=" + values
-            else:
-                override =""
 
-            if self.inputFlag:
-                nameVal = self.getInputs()
-                for n in nameVal:
-                    tupleList = nameVal.get(n)
+        if self.xmlFile is None:
+            return print("Linearization cannot be performed as the model is not build, use ModelicaSystem() to build the model first")
+
+        overrideLinearFile = os.path.join(self.tempdir, '{}.{}'.format(self.modelName + "_override_linear", "txt")).replace("\\", "/")
+
+        file = open(overrideLinearFile, "w")
+        for (key, value) in self.overridevariables.items():
+            name = key + "=" + value + "\n"
+            file.write(name)
+        for (key, value) in self.linearOptions.items():
+            name = key + "=" + str(value) + "\n"
+            file.write(name)
+        file.close()
+
+        override =" -overrideFile=" + overrideLinearFile
+        # print(override)
+
+        if self.inputFlag:
+            nameVal = self.getInputs()
+            for n in nameVal:
+                tupleList = nameVal.get(n)
+                if tupleList is not None:
                     for l in tupleList:
                         if l[0] < float(self.simulateOptions["startTime"]):
                             print('Input time value is less than simulation startTime')
                             return
-                self.__simInput()
-                csvinput ="-csvInput=" + self.csvFile
+            self.createCSVData()
+            csvinput =" -csvInput=" + self.csvFile
+        else:
+            csvinput=""
+
+        ## prepare the linearization runtime command
+        if (platform.system() == "Windows"):
+            getExeFile = os.path.join(self.tempdir, '{}.{}'.format(self.modelName, "exe")).replace("\\", "/")
+        else:
+            getExeFile = os.path.join(self.tempdir, self.modelName).replace("\\", "/")
+
+        if lintime is None:
+            linruntime = " -l=" + str(self.linearOptions["stopTime"])
+        else:
+            linruntime = " -l=" + lintime
+
+        if simflags is None:
+            simflags = ""
+
+        currentDir = os.getcwd()
+        if (os.path.exists(getExeFile)):
+            cmd = getExeFile + linruntime + override + csvinput + simflags
+            # print(cmd)
+            os.chdir(self.tempdir)
+            if (platform.system() == "Windows"):
+                omhome = os.path.join(os.environ.get("OPENMODELICAHOME"))
+                dllPath = os.path.join(omhome, "bin").replace("\\", "/") + os.pathsep + os.path.join(omhome, "lib/omc").replace("\\", "/") + os.pathsep + os.path.join(omhome, "lib/omc/cpp").replace("\\", "/") +  os.pathsep + os.path.join(omhome, "lib/omc/omsicpp").replace("\\", "/")
+                my_env = os.environ.copy()
+                my_env["PATH"] = dllPath + os.pathsep + my_env["PATH"]
+                p = subprocess.Popen(cmd, env=my_env)
+                p.wait()
+                p.terminate()
             else:
-                csvinput=""
+                os.system(cmd)
+        else:
+            os.chdir(currentDir)
+            raise Exception("Error: Application file path not found: " +  getExeFile)
 
-            #linexpr="linearize(" + self.modelName + "," + properties + ", simflags=\" " + csvinput + " " + override + " \")"
-            self.getconn.sendExpression("linearize(" + self.modelName + "," + properties + ", simflags=\" " + csvinput + " " + override + " \")")
-            linearizeError = ''
-            linearizeError = self.requestApi('getErrorString')
-            if linearizeError:
-                print(linearizeError)
-                return
+        # code to get the matrix and linear inputs, outputs and states
+        linearFile = os.path.join(self.tempdir, "linearized_model.py").replace("\\","/")
+
+        # support older openmodelica versions before OpenModelica v1.16.2 where linearize() generates "linear_modelname.mo" file
+        if not os.path.exists(linearFile):
+            linearFile = '{}_{}.{}'.format('linear', self.modelName, 'py')
+
+        if os.path.exists(linearFile):
+            # this function is called from the generated python code linearized_model.py at runtime,
+            # to improve the performance by directly reading the matrices A, B, C and D from the julia code and avoid building the linearized modelica model
+            try:
+                ## add the generated linearfile directory to system path, as running from script does not find the module
+                ## https://github.com/OpenModelica/OMPython/issues/196
+                sys.path.append(os.path.dirname(linearFile))
+                from linearized_model import linearized_model
+                result = linearized_model()
+                (n, m, p, x0, u0, A, B, C, D, stateVars, inputVars, outputVars) = result
+                self.linearinputs = inputVars
+                self.linearoutputs = outputVars
+                self.linearstates = stateVars
+                return [A, B, C, D]
+                os.chdir(currentDir)
+            except:
+                os.chdir(currentDir)
+                raise Exception("ModuleNotFoundError: No module named 'linearized_model'")
+        else:
+            errormsg = self.getconn.sendExpression("getErrorString()")
+            os.chdir(currentDir)
+            return print("Linearization failed: ", "\"" , linearFile,"\"" ," not found \n", errormsg)
 
-            # code to get the matrix and linear inputs, outputs and states
-            getLinFile = '{}_{}.{}'.format('linear', self.modelName, 'mo')
-            checkLinFile = os.path.exists(getLinFile)
-            if checkLinFile:
-                self.requestApi('loadFile', getLinFile)
-                cNames = self.requestApi('getClassNames')
-                linModelName = cNames[0]
-                buildModelmsg=self.requestApi('buildModel', linModelName)
-                self.xmlFile=os.path.join(os.path.dirname(buildModelmsg[0]),buildModelmsg[1]).replace("\\","/")
-                if(os.path.exists(self.xmlFile)):
-                    self.linearizationFlag = True
-                    self.linearparameters={}
-                    self.linearquantitiesList=[]
-                    self.linearinputs=[]
-                    self.linearoutputs=[]
-                    self.linearstates=[]
-                    self.xmlparse()
-                    matrices = self.getlinearMatrix()
-                    return matrices
-                else:
-                    return self.requestApi('getErrorString')
-        except Exception as e:
-            raise e
 
     def getLinearInputs(self):
         """
@@ -1693,49 +1801,6 @@ class ModelicaSystem(object):
         """
         return self.linearstates
 
-    def getlinearMatrix(self):
-        """
-        Helper Function which generates the Linear Matrix A,B,C,D
-        """
-        matrix_A=OrderedDict()
-        matrix_B=OrderedDict()
-        matrix_C=OrderedDict()
-        matrix_D=OrderedDict()
-        for i in self.linearparameters:
-            name=i
-            if(name[0]=="A"):
-                matrix_A[name]=self.linearparameters[i]
-            if(name[0]=="B"):
-                matrix_B[name]=self.linearparameters[i]
-            if(name[0]=="C"):
-                matrix_C[name]=self.linearparameters[i]
-            if(name[0]=="D"):
-                matrix_D[name]=self.linearparameters[i]
-
-        tmpmatrix_A = self.getLinearMatrixValues(matrix_A)
-        tmpmatrix_B = self.getLinearMatrixValues(matrix_B)
-        tmpmatrix_C = self.getLinearMatrixValues(matrix_C)
-        tmpmatrix_D = self.getLinearMatrixValues(matrix_D)
-
-        return [tmpmatrix_A,tmpmatrix_B,tmpmatrix_C,tmpmatrix_D]
-
-    def getLinearMatrixValues(self,matrix):
-        """
-        Helper Function which generates the Linear Matrix A,B,C,D
-        """
-        if (matrix):
-            x=list(matrix.keys())
-            name=x[-1]
-            tmpmatrix=np.zeros((int(name[2]),int(name[4])))
-            for i in x:
-                rows=int(i[2])-1
-                cols=int(i[4])-1
-                tmpmatrix[rows][cols]=matrix[i]
-            return tmpmatrix
-        else:
-            return np.zeros((0,0))
-
-
 def FindBestOMCSession(*args, **kwargs):
   """
   Analyzes the OMC executable version string to find a suitable selection
